# -*- coding: utf-8 -*-
"""
This is only to show the progress on a window visible to the user
"""
import os

from PyQt5 import uic
from PyQt5 import QtWidgets
from qgis.PyQt.QtWidgets import QPlainTextEdit
from .ExceptionReporting import ExceptionReporting

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'issueModule_dialog_base.ui'))


class IssueClassDialog(QtWidgets.QDialog, FORM_CLASS):
    def __init__(self, parent=None):
        self._bufferedLine = u""
        super(IssueClassDialog, self).__init__(parent)
        self.setupUi(self)

    def write(self, msg):
        self._bufferedLine = u"{}{}".format(self._bufferedLine, msg)
        if u'\n' in self._bufferedLine:
            if self._bufferedLine[-1] == u'\n':
                self._bufferedLine = self._bufferedLine[:-1]
            self._flushParagraph()
    def _flushParagraph(self):
        if self._bufferedLine:
            self.ui_logger.appendPlainText(self._bufferedLine)
            self._bufferedLine = u""
            self.ui_logger.ensureCursorVisible()
    
    def _close(self):
        self._flushParagraph()

    def run(self, function):
        self._bufferedLine = u""
        self._show()
        try:
            function()
        except Exception as e:
            tb = ExceptionReporting.getTraceback()
            tracebackLines = ExceptionReporting.getTracebackAsString(e)
            try:
                self.write(tracebackLines)
            except:
                pass
            raise e.with_traceback(tb)
        finally:
            self._close()
        
    def _show(self):
        ui_logger = self.ui_logger #: :type ui_logger: QPlainTextEdit 
        ui_logger.setPlainText(u"")
        ui_logger.setLineWrapMode(QPlainTextEdit.WidgetWidth)
        self.adjustSize()
        super(IssueClassDialog, self).show()
        self.show()

