# -*- coding: utf-8 -*-
"""
Here we isolate the logic to replay the issue
"""

from qgis._core import QgsProject, QgsVectorLayer, QgsFeature, QgsWkbTypes,\
    QgsPoint, QgsGeometry
import os
import inspect
class IssueReplay():
    
    def __init__(self, log):
        self.log = log
        
    def write(self, msg):
        self.log.write(msg) 
               
    def replayIssue(self):
        inputLayer = self.ensureVectorLayerLoaded()
        for i in range(1,800):
            self.write("Iteration #{}\n".format(i))
            self._extractNodes(inputLayer)

    def ensureVectorLayerLoaded(self):
        """ Ensure the sample shapefile is loaded, and returns its ID """
        inputLayer = self.getLayerByName("polygon_utm30")
        if not inputLayer:
            thisPath = os.path.dirname(inspect.getfile(self.__class__))
            source = self.sampleShapePath = os.path.join(thisPath, "sample_shape", "polygon_utm30.shp")
            inputLayer = QgsVectorLayer(source, "polygon_utm30", "ogr")
            QgsProject.instance().addMapLayer(inputLayer)
        return inputLayer

    def getLayerByName(self, layerName):
        """ Finds a layer by name """
        foundLayers = QgsProject.instance().mapLayersByName(layerName)
        return foundLayers[0] if foundLayers else None

    def _extractNodes(self, layer):
        
        for ft in layer.getFeatures():
            geom = ft.geometry()
            geomV2 = geom.constGet()
            if geom.type() == QgsWkbTypes.PolygonGeometry :
                if isinstance(geomV2, QgsPoint):
                    self.write("WHAT??????!!!!!   Type is really PolygonGeometry, but we get a QgsPoint???... That's going to break something....")
                for i in range(0, geomV2.numGeometries()):
                    polygonV2 = geomV2.geometryN(i)
                    lineStringV2 = polygonV2.exteriorRing()
                    for i in range(0, lineStringV2.numPoints()): 
                        nodo = lineStringV2.pointN(i)

                        ft2 = QgsFeature(ft.fields())
                        REPLAY_BUG = True
                        if REPLAY_BUG:
                            # This eventually ends with the issue showing up
                            ft2.setGeometry(QgsGeometry(nodo.clone()))
                        else:
                            ft2.setGeometry(QgsGeometry(QgsPoint(nodo.x(), nodo.y())))

        return None
